/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.model;


import java.util.ArrayList;
import java.util.List;

/**
 * Se comporte comme le string tokenizer Java � lexception qu'il n'ignore pas
 * les tokens vides.<br>
 * Exemple:
 * 	<code>new StringTokenizer(";a;;bb;ccc;", ";") --> ["a", "bb", "ccc"]</code><br>
 * 	<code>new ExactStringTokenizer(";a;;bb;ccc;", ";") --> ["", "a", "", "bb", "ccc", ""]</code>
 */
public class ExactStringTokenizer
{
	private static final String[] STRING_ARRAY = {};
	
	public static final String[] split(String iString, char delim)
	{
		List<String> ret = new ArrayList<String>();
		ExactStringTokenizer st = new ExactStringTokenizer(iString, delim);
		
		while(st.hasMoreTokens()){
			ret.add(st.nextToken());
		}
		
		return ret.toArray(STRING_ARRAY);
	}
	
	private String str;
	private int curTokenPos;
	private int nextDelimPos;
	private char delim;
	
	public ExactStringTokenizer(String iString, char delim)
	{
		str = iString;
		this.delim = delim;
		reinit();
	}
	private void reinit()
	{
		curTokenPos=0;
		nextDelimPos = Integer.MIN_VALUE;
	}
	public boolean hasMoreTokens()
	{
		if(nextDelimPos == Integer.MIN_VALUE){
			nextDelimPos = str.indexOf(delim, curTokenPos);
		}
		
		return nextDelimPos >= 0 || curTokenPos <= str.length();
	}
	public String nextToken()
	{
		if(!hasMoreTokens()){
			return null;
		}
		
		String ret = null;
		if(nextDelimPos >= 0)
		{
			ret = str.substring(curTokenPos, nextDelimPos);
			curTokenPos = nextDelimPos + 1;
			nextDelimPos = Integer.MIN_VALUE;
		}
		else
		{
			// --- end of string
			ret = str.substring(curTokenPos);
			curTokenPos = str.length()+1;
		}
		return ret;
	}
}
