/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.model.dataaccess;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Locale;
import java.util.Random;

import org.ow2.opensuit.samples.livetour.model.ExactStringTokenizer;
import org.ow2.opensuit.samples.livetour.model.to.MovieTO;
import org.ow2.opensuit.samples.livetour.model.to.PersonTO;



/**
 * This class gives acces to the movies database.
 */
public class MoviesDataBase
{
	private static final char SEPAR = '\t';
	private static final PersonTO[] PERSON_ARRAY = {};
	private static final MovieTO[] MOVIE_ARRAY = {};
	private static MoviesDataBase database;
	
	/**
	 * Returns the database connection object.
	 * @return
	 * @throws DataBaseError In case of a technical error.
	 */
	public static MoviesDataBase getDataBase() throws DataBaseError
	{
		if(database == null)
		{
			database = new MoviesDataBase();
			database.load();
		}
		return database;
	}
	
	
	private ArrayList<PersonTO> directors;
	private ArrayList<MovieTO> movies;
	
	private final DateFormat YEAR_FORMAT = new SimpleDateFormat("yyyy");
	private final DateFormat DATE_FORMAT = new SimpleDateFormat("d MMMMM yyyy", Locale.US);
	
	private static File workDir;
	static
	{
		workDir = new File(System.getProperty("user.home"));
		
		if(!workDir.exists()){
			workDir = new File(System.getProperty("java.io.tmpdir"));
		}
	}

	private MoviesDataBase()
	{
//		load();
	}
	private void load()
	{
		try
		{
			loadDirectors();
			loadMovies();
		}
		catch(IOException ioe)
		{
			ioe.printStackTrace();
		}
	}
	private InputStream readFile(String name) throws FileNotFoundException
	{
		File workFile = new File(workDir, name);
		if(workFile.exists())
		{
			System.out.println("read '"+name+"' from: "+workFile.getAbsolutePath());
			return new FileInputStream(workFile);
		}
		
		System.out.println("read '"+name+"' from resources");
		return MoviesDataBase.class.getClassLoader().getResourceAsStream(name);
	}
	private OutputStream writeFile(String name) throws FileNotFoundException
	{
		File workFile = new File(workDir, name);
		System.out.println("save '"+name+"' to: "+workFile.getAbsolutePath());
		return new FileOutputStream(workFile);
	}
	private void loadDirectors() throws IOException
	{
		// --- 1: load directors
		directors = new ArrayList<PersonTO>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(readFile("directors.txt")));
		try
		{
			String curline = null;
			while((curline = reader.readLine()) != null)
			{
//				curline = curline.trim();
				if(curline.length() == 0 || curline.startsWith("#")){
					// skip
					continue;
				}
				
				// --- create a new director
//				String[] values = curline.split("\t");
				String[] values = ExactStringTokenizer.split(curline, SEPAR);
				PersonTO director = new PersonTO();
				director.setId(Integer.parseInt(values[0]));
				director.setName(values[1]);
				director.setNationality(values[2]);
				
				if(values[3].length() > 0){
					try
	                {
		                director.setBirth(DATE_FORMAT.parse(values[3]));
	                }
	                catch(ParseException e)
	                {
		                // TODO Auto-generated catch block
		                e.printStackTrace();
	                }
				}
				
				if(values[4].length() > 0){
	                try
                    {
	                    director.setDeath(DATE_FORMAT.parse(values[4]));
                    }
                    catch(ParseException e)
                    {
	                    // TODO Auto-generated catch block
	                    e.printStackTrace();
                    }
				}
				
				directors.add(director);
			}
		}
		finally
		{
			reader.close();
		}
	}
	
	private void loadMovies() throws IOException
	{
		// --- 2: load movies
		movies = new ArrayList<MovieTO>();
		BufferedReader reader = new BufferedReader(new InputStreamReader(readFile("movies.txt")));
		try
		{
			String curline = null;
			while((curline = reader.readLine()) != null)
			{
//				curline = curline.trim();
				if(curline.length() == 0 || curline.startsWith("#")){
					// skip
					continue;
				}
				
				// --- create a new movie
//				String[] values = curline.split("\t");
				String[] values = ExactStringTokenizer.split(curline, SEPAR);
				MovieTO movie = new MovieTO();
				movie.setId(Integer.parseInt(values[0]));
				movie.setFrenchTitle(values[1]);
				movie.setOriginalTitle(values[2]);
				
				if(values[3].length() > 0){
					try
	                {
		                movie.setReleaseDate(YEAR_FORMAT.parse(values[3]));
	                }
	                catch(ParseException e)
	                {
		                // TODO Auto-generated catch block
		                e.printStackTrace();
	                }
				}
				
                int directorId = Integer.parseInt(values[4]);
				try
                {
	                movie.setDirector(getDirector(directorId));
                }
                catch(DataBaseError e)
                {
	                // TODO Auto-generated catch block
	                e.printStackTrace();
                }
				
				movies.add(movie);
			}
		}
		finally
		{
			reader.close();
		}
	}
	private void saveDirectors()
	{
		PrintWriter writer = null;
        try
        {
	        writer = new PrintWriter(writeFile("directors.txt"));
        }
        catch(FileNotFoundException e)
        {
	        e.printStackTrace();
	        return;
        }
		
		writer.println("# Directors");
		for(int i=0; i<directors.size(); i++)
		{
			PersonTO director = directors.get(i);
			writer.print(String.valueOf(director.getId()));
			writer.print(SEPAR);
			writer.print(director.getName());
			writer.print(SEPAR);
			writer.print(director.getNationality());
			writer.print(SEPAR);
			writer.print(DATE_FORMAT.format(director.getBirth()));
			writer.print(SEPAR);
			
			if(director.getDeath() != null){
				writer.print(DATE_FORMAT.format(director.getDeath()));
			}
			
			writer.println();
		}
		
		writer.close();
	}
	private void saveMovies()
	{
		PrintWriter writer = null;
        try
        {
	        writer = new PrintWriter(writeFile("movies.txt"));
        }
        catch(FileNotFoundException e)
        {
	        e.printStackTrace();
	        return;
        }
		
		writer.println("# Movies");
		for(int i=0; i<movies.size(); i++)
		{
			MovieTO movie = movies.get(i);
			writer.print(String.valueOf(movie.getId()));
			writer.print(SEPAR);
			writer.print(movie.getFrenchTitle());
			writer.print(SEPAR);
			writer.print(movie.getOriginalTitle());
			writer.print(SEPAR);
			
			if(movie.getReleaseDate() != null){
				writer.print(YEAR_FORMAT.format(movie.getReleaseDate()));
			}
			
			writer.print(SEPAR);
			writer.print(String.valueOf(movie.getDirector().getId()));
			writer.println();
		}
		
		writer.close();
	}
	// ========================================================================================
	// === Movies Methods
	// ========================================================================================
	private int getMovieIndex(int iId)
	{
		for(int i=0; i<movies.size(); i++)
		{
			if(movies.get(i).getId() == iId){
				return i;
			}
		}
		return -1;
	}
	/**
	 * Returns the movie description from its ID
	 * @param iId
	 * @return
	 * @throws DataBaseError This method may either throw a TECHNICAL_ERROR in case of a 
	 * technical error (communication failure, ...), or DATA_NOT_FOUND_ERROR if no
	 * movie was found in the database with the specified ID.
	 */
	public MovieTO getMovie(int iId) throws DataBaseError
	{
		int idx = getMovieIndex(iId);
		
		if(idx < 0){
			throw new DataBaseError(DataBaseError.DATA_NOT_FOUND_ERROR, "Movie "+iId+" not found.");
		}
		
		try {
            return movies.get(idx).clone();
        }  catch (final CloneNotSupportedException e) {
            throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
        }
	}
	/**
	 * Counts all movies
	 * @return
	 */
	public int countAllMovies() throws DataBaseError
	{
		return movies.size();
	}
	private MovieTO[] applyRequestOptions(ArrayList<MovieTO> list, RequestOptions iOptions)
	{
		MovieTO[] ret = list.toArray(MOVIE_ARRAY);
		
		if(iOptions != null)
		{
			// --- 1: sort
			if(iOptions.hasSortOption()){
				Arrays.sort(ret, new MoviesComparator(iOptions.getSortCriteria(), iOptions.isSortAscendingly()));
			}
			
			// --- 2: paginate
			if(iOptions.hasPagination())
			{
				MovieTO[] paginated = new MovieTO[Math.min(iOptions.getNbOfResults(), ret.length-iOptions.getFirstResult())];
				System.arraycopy(ret, iOptions.getFirstResult(), paginated, 0, paginated.length);
				ret = paginated;
			}
		}
		
		return ret;
	}
	/**
	 * Returns all movies
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public MovieTO[] getAllMovies(RequestOptions iOptions) throws DataBaseError
	{
		return applyRequestOptions(movies, iOptions);
	}
	private int createMovieId()
	{
		Random rd = new Random();
		while(true)
		{
//			String id = String.valueOf(Math.abs(rd.nextInt()));
			int id = Math.abs(rd.nextInt());
			
            if(getMovieIndex(id) < 0){
            	return id;
            }
		}
	}
	/**
	 * Sets the given movie.<br>
	 * This method either overwrites an existing movie or creates a new movie in the database.
	 * @param iMovie
	 */
	public void setMovie(MovieTO iMovie) throws DataBaseError
	{
		int idx = iMovie.getId() == 0 ? -1 : getMovieIndex(iMovie.getId());
		if(idx < 0)
		{
			// --- add a new movie
			iMovie.setId(createMovieId());
			try {
                iMovie = iMovie.clone();
            } catch (final CloneNotSupportedException e) {
                throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
            }
            
			movies.add(iMovie);
		}
		else
		{
			// --- update an existing movie
			try {
                iMovie = iMovie.clone();
            }  catch (final CloneNotSupportedException e) {
                throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
            }
            
			movies.set(idx, iMovie);
		}
		saveMovies();
	}
	/**
	 * Removes the given movie from the database.
	 * @param iId
	 */
	public void removeMovie(int iId) throws DataBaseError
	{
		int idx = getMovieIndex(iId);
		
		if(idx < 0){
			return;
		}
		
		movies.remove(idx);
		saveMovies();
	}
	/**
	 * Searches movies with title
	 * @param title
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public MovieTO[] searchMovieByTitle(String title, RequestOptions iOptions) throws DataBaseError
	{
		// --- 1: search
		title = title.toLowerCase();
		
		ArrayList<MovieTO> results = new ArrayList<MovieTO>();
		for(int i=0; i<movies.size(); i++)
		{
			MovieTO movie = movies.get(i);
			if(movie.getFrenchTitle().toLowerCase().contains(title) || movie.getOriginalTitle().toLowerCase().contains(title))
			{
				results.add(movie);
			}
		}
		
		// --- 2: process options
		return applyRequestOptions(results, iOptions);
	}
	public int countMovieByTitle(String title) throws DataBaseError
	{
		int count = 0;
		for(int i=0; i<movies.size(); i++)
		{
			MovieTO movie = movies.get(i);
			if(movie.getFrenchTitle().toLowerCase().contains(title) || movie.getOriginalTitle().toLowerCase().contains(title))
			{
				count++;
			}
		}
		return count;
	}
	/**
	 * Searches movies by director name
	 * @param director
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public MovieTO[] searchMoviesByDirector(String director, RequestOptions iOptions) throws DataBaseError
	{
		PersonTO[] matchingDirectors = searchDirectorByName(director, null);
		
		ArrayList<MovieTO> results = new ArrayList<MovieTO>();
		if(matchingDirectors != null && matchingDirectors.length > 0)
		{
			for(int i=0; i<matchingDirectors.length; i++)
			{
				MovieTO[] movies = getMoviesFromDirector(matchingDirectors[i].getId(), null);
				
				for(int j=0; j<movies.length; j++){
					results.add(movies[j]);
				}
			}
		}
		
		// --- 2: process options
		return applyRequestOptions(results, iOptions);
	}
	public int countMoviesByDirector(String director) throws DataBaseError
	{
		PersonTO[] matchingDirectors = searchDirectorByName(director, null);
		
		int count = 0;
		if(matchingDirectors != null && matchingDirectors.length > 0)
		{
			for(int i=0; i<matchingDirectors.length; i++)
			{
				MovieTO[] movies = getMoviesFromDirector(matchingDirectors[i].getId(), null);
				count += movies.length;
			}
		}
		return count;
	}
	/**
	 * Returns all movies from the given director
	 * @param iDirectorId
	 * @return
	 */
	public MovieTO[] getMoviesFromDirector(int iDirectorId, RequestOptions iOptions) throws DataBaseError
	{
		ArrayList<MovieTO> results = new ArrayList<MovieTO>();
		for(int i=0; i<movies.size(); i++)
		{
			MovieTO movie = movies.get(i);
			if(movie.getDirector().getId() == iDirectorId)
			{
				results.add(movie);
			}
		}
		return applyRequestOptions(results, iOptions);
	}
	// ==============================================================================
	// === Movies Comparator (to implement sorting)
	// ==============================================================================
	/**
	 * Sort criteria: sort by ID (for movie and director requests)
	 */
	public static final int SORT_BY_ID = 1;
	/**
	 * Sort criteria: sort by french title (for movie requests)
	 */
	public static final int SORT_BY_FRENCH_TITLE = 2;
	/**
	 * Sort criteria: sort by original title (for movie requests)
	 */
	public static final int SORT_BY_ORIGINAL_TITLE = 3;
	/**
	 * Sort criteria: sort by director name (for movie requests)
	 */
	public static final int SORT_BY_DIRECTOR = 4;
	/**
	 * Sort criteria: sort by relesae date (for movie requests)
	 */
	public static final int SORT_BY_RELEASE_DATE = 5;
	
	private static class MoviesComparator implements Comparator<MovieTO>
	{
		private int crit;
		private int sign;
		
		public MoviesComparator(int iSortingCriteria, boolean iSortAscendingly)
		{
			crit = iSortingCriteria;
			sign = iSortAscendingly ? 1 : -1;
		}
		public int compare(MovieTO m1, MovieTO m2)
		{
			switch(crit)
			{
				case SORT_BY_ID:
				    
					return sign*(m1.getId()-m2.getId());
					
				case SORT_BY_FRENCH_TITLE:
				    
					if(m1.getFrenchTitle() == null)
					{
						if(m2.getFrenchTitle() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getFrenchTitle() == null){
						return sign;
					}
					
					return sign * m1.getFrenchTitle().compareTo(m2.getFrenchTitle());
					
				case SORT_BY_DIRECTOR:
				    
					if(m1.getDirector() == null)
					{
					    
						if(m2.getDirector() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getDirector() == null){
						return sign;
					}
					
					return sign * m1.getDirector().getName().compareTo(m2.getDirector().getName());
					
				case SORT_BY_RELEASE_DATE:
				    
					if(m1.getReleaseDate() == null)
					{
						
					    if(m2.getReleaseDate() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getReleaseDate() == null){
						return sign;
					}
					
					return sign * m1.getReleaseDate().compareTo(m2.getReleaseDate());
					
				default: // SORT_BY_ORIGINAL_TITLE:
				    
					if(m1.getOriginalTitle() == null)
					{
						if(m2.getOriginalTitle() == null){
							return 0;
						}
						
						return -sign;
					}
					
					if(m2.getOriginalTitle() == null){
						return sign;
					}
					
					return sign * m1.getOriginalTitle().compareTo(m2.getOriginalTitle());
			}
		}
	}
	// ========================================================================================
	// === Directors Methods
	// ========================================================================================
	private int getDirectorIndex(int iId)
	{
		for(int i=0; i<directors.size(); i++)
		{
			if(directors.get(i).getId() == iId){
				return i;
			}
		}
		return -1;
	}
	private PersonTO[] applyRequestOptions2(ArrayList<PersonTO> list, RequestOptions iOptions)
	{
		PersonTO[] ret = list.toArray(PERSON_ARRAY);
		
		if(iOptions != null)
		{
			// --- 1: sort
			if(iOptions.hasSortOption()){
				Arrays.sort(ret, new PersonComparator(iOptions.getSortCriteria(), iOptions.isSortAscendingly()));
			}
			
			// --- 2: paginate
			if(iOptions.hasPagination())
			{
				PersonTO[] paginated = new PersonTO[Math.min(iOptions.getNbOfResults(), ret.length-iOptions.getFirstResult())];
				System.arraycopy(ret, iOptions.getFirstResult(), paginated, 0, paginated.length);
				ret = paginated;
			}
		}
		
		return ret;
	}
	/**
	 * Returns the director description from its id
	 * @param iId
	 * @return
	 * @throws DataBaseError This method may either throw a TECHNICAL_ERROR in case of a 
	 * technical error (communication failure, ...), or DATA_NOT_FOUND_ERROR if no
	 * director was found in the database with the specified ID.
	 */
	public PersonTO getDirector(int iId) throws DataBaseError
	{
		int idx = getDirectorIndex(iId);
		
		if(idx < 0){
			throw new DataBaseError(DataBaseError.DATA_NOT_FOUND_ERROR, "Director "+iId+" not found.");
		}
		
		try {
            return directors.get(idx).clone();
        } catch (final CloneNotSupportedException e) {
            throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
        }
	}
	/**
	 * Returns all directors
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public PersonTO[] getAllDirectors(RequestOptions iOptions) throws DataBaseError
	{
		return applyRequestOptions2(directors, iOptions);
	}
	/**
	 * Counts all directors
	 * @return
	 */
	public int countAllDirectors() throws DataBaseError
	{
		return directors.size();
	}
	private int createDirectorId()
	{
		Random rd = new Random();
		while(true)
		{
//			String id = String.valueOf(Math.abs(rd.nextInt()));
			int id = Math.abs(rd.nextInt());
            
			if(getDirectorIndex(id) < 0){
            	return id;
            }
		}
	}
	/**
	 * Sets the given director.<br>
	 * This method either overwrites an existing director or creates a new one in the database.
	 * @param iMovie
	 */
	public void setDirector(PersonTO iDirector) throws DataBaseError
	{
		try {
            iDirector = iDirector.clone();
        } catch (final CloneNotSupportedException e) {
            throw new DataBaseError(DataBaseError.TECHNICAL_ERROR, e.getMessage());
        }
        
		int idx = iDirector.getId() == 0 ? -1 : getDirectorIndex(iDirector.getId());
		if(idx < 0)
		{
			// --- add a new movie
			iDirector.setId(createDirectorId());
			directors.add(iDirector);
		}
		else
		{
			// --- update an existing movie
			directors.set(idx, iDirector);
		}
		saveDirectors();
	}
	/**
	 * Removes the director with specified id from database
	 * @param iId
	 */
	public void removeDirector(int iId) throws DataBaseError
	{
		int idx = getDirectorIndex(iId);
		
		if(idx < 0){
			return;
		}
		
		movies.remove(idx);
		saveDirectors();
	}
	/**
	 * Searches director by name
	 * @param director
	 * @param iOptions Specifies request options (sort, pagination). May be null.
	 * @return
	 */
	public PersonTO[] searchDirectorByName(String name, RequestOptions iOptions) throws DataBaseError
	{
		name = name.toLowerCase();
		
		ArrayList<PersonTO> results = new ArrayList<PersonTO>();
		for(int i=0; i<directors.size(); i++)
		{
			PersonTO director = directors.get(i);
			if(director.getName().toLowerCase().contains(name))
			{
				results.add(director);
			}
		}
		return applyRequestOptions2(results, iOptions);
	}
	// ==============================================================================
	// === Person Comparator (to implement sorting)
	// ==============================================================================
	/**
	 * Sort criteria: sort by name (for director requests)
	 */
	public static final int SORT_BY_NAME = 2;
	/**
	 * Sort criteria: sort by nationality (for director requests)
	 */
	public static final int SORT_BY_NATIONALITY = 3;
	/**
	 * Sort criteria: sort by birth date (for director requests)
	 */
	public static final int SORT_BY_BIRTH_DATE = 4;
	/**
	 * Sort criteria: sort by death date (for director requests)
	 */
	public static final int SORT_BY_DEATH_DATE = 5;
	
	private static class PersonComparator implements Comparator<PersonTO>
	{
		private int crit;
		private int sign;
		
		public PersonComparator(int iSortingCriteria, boolean iSortAscendingly)
		{
			crit = iSortingCriteria;
			sign = iSortAscendingly ? 1 : -1;
		}
		public int compare(PersonTO m1, PersonTO m2)
		{
			switch(crit)
			{
				case SORT_BY_ID:
				    
					return sign*(m1.getId()-m2.getId());
					
				case SORT_BY_NATIONALITY:
				    
					if(m1.getNationality() == null)
					{
						
					    if(m2.getNationality() == null){
							return 0;
						}
						
						return sign;
					}
					
					if(m2.getNationality() == null){
						return sign;
					}
					
					return sign * m1.getNationality().compareTo(m2.getNationality());
					
				case SORT_BY_BIRTH_DATE:
				    
					if(m1.getBirth() == null)
					{
						if(m2.getBirth() == null){
							return 0;
						}
						
						return sign;
					}
					
					if(m2.getBirth() == null){
						return sign;
					}
					
					return sign * m1.getBirth().compareTo(m2.getBirth());
					
				case SORT_BY_DEATH_DATE:
				    
					if(m1.getDeath() == null)
					{
						if(m2.getDeath() == null){
							return 0;
						}
						
						return sign;
					}
					
					if(m2.getDeath() == null){
						return sign;
					}
					
					return sign * m1.getDeath().compareTo(m2.getDeath());
					
				default: // SORT_BY_NAME:
				    
					if(m1.getName() == null)
					{
						if(m2.getName() == null){
							return 0;
						}
						
						return sign;
					}
					
					if(m2.getName() == null){
						return sign;
					}
					
					return sign * m1.getName().compareTo(m2.getName());
			}
		}
	}	
}
