/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.uibeans;

import java.io.Serializable;

import org.ow2.opensuit.core.error.LocalizedError;
import org.ow2.opensuit.samples.livetour.model.dataaccess.DataBaseError;
import org.ow2.opensuit.samples.livetour.model.dataaccess.MoviesDataBase;
import org.ow2.opensuit.samples.livetour.model.to.MovieTO;



/**
 * This bean holds a list of movies.
 * 
 * It initializes itself by loading all movies from the database.
 * 
 * <p>This version of the bean performs the search when loading, then
 * lets the framework manage sort and pagination.
 * This design may only be used if you're sure the amount of data is "reasonable"
 * and under control.
 * <p>See SearchMovieBean2 for a dynamic version of it (that have the model
 * layer manage sorting and pagination).
 */
public class ListOfMoviesBean implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	// ==========================================================
	// === Attributes
	// ==========================================================
	private MovieTO[] movies;
	
	// ==========================================================
	// === Control Methods
	// ==========================================================
	/**
	 * Loads all movies from the database
	 */
	public void loadAll() throws LocalizedError
	{
		try
        {
	        movies = MoviesDataBase.getDataBase().getAllMovies(null);
        }
        catch(DataBaseError e)
        {
			throw new LocalizedError("database_technical", new Object[]{e.getMessage()});
        }
	}
	// ==========================================================
	// === Attributes Getters & Setters
	// ==========================================================
	public MovieTO[] getMovies()
    {
    	return movies;
    }
	
}
