/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.uibeans;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.ow2.opensuit.core.validation.LocalizedValidationError;
import org.ow2.opensuit.core.validation.ValidationErrors;


public class PersonBean implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	private static int[] ALL_GENDERS = {0, 1, 2};
	private static int[] ALL_ACHIEVEMENTS = {0, 1, 2, 3, 4};
	
	// === Civility
	// onchange
	private int gender;
	// autocomplete (?)
	private String firstname;
	private String lastname;
	// visible or not depending on the gender
	private String maidenname;
	
	// === Misc
	private int[] achievements;
	
	// === Birth
	// validation
	private Date birthdate;
	private int birthsize;
	private int birthweight;
	// autocompletion
	private String birthcountry;
	// autocompletion (?)
	private String birthcity;
	
	// === Death
	private boolean dead;
	// validation
	private Date deathdate;
	// autocompletion
	private String deathcountry;
	// autocompletion (?)
	private String deathcity;
	
	private static String[] allCountries;
	static
	{
		String[] countryCodes = Locale.getISOCountries();
		allCountries = new String[countryCodes.length];
//		allCountriesLC = new String[countryCodes.length];
		for(int i=0; i<countryCodes.length; i++)
		{
			allCountries[i] = new Locale("en", countryCodes[i]).getDisplayCountry(Locale.ENGLISH);
//			allCountriesLC[i] = allCountries[i].toLowerCase();
		}
	}

	// ==========================================================
	// === Control Methods
	// ==========================================================
	/**
	 * Initializes this bean to create a new person
	 */
	public void init()
	{
		gender = 0;
		firstname = null;
		lastname = null;
		maidenname = null;
		birthdate = null;
		birthsize = 50;
		birthweight = 3500;
		birthcountry = null;
		birthcity = null;
		
		dead = false;
		deathdate = null;
		deathcountry = null;
		deathcity = null;
	}
	/**
	 * Reloads the form
	 */
	public void reload()
	{
	}
	/**
	 * Performs code validation
	 * @throws ValidationErrors
	 */
	public void validate() throws ValidationErrors
	{
		ValidationErrors errors = new ValidationErrors();
		if(gender == 0)
		{
			// --- a gender has to be selected
			errors.addItemError("person.gender", new LocalizedValidationError("validation.required"));
		}
		if(birthdate != null)
		{
			// --- check it is in the past
			Date now = new Date();
			
			if(birthdate.after(now)){
				errors.addItemError("person.birthdate", new LocalizedValidationError("validation.date_not_past"));
			}
		}
		if(dead)
		{
			if(deathdate == null)
			{
				errors.addItemError("person.deathdate", new LocalizedValidationError("validation.required"));
			}
			else
			{
				// --- check it is in the past
				Date now = new Date();
				
				if(deathdate.after(now)){
					errors.addItemError("person.deathdate", new LocalizedValidationError("validation.date_not_past"));
				}
				
				// --- finally check the death is after the birth
				if(birthdate != null && deathdate.before(birthdate)){
					errors.addItemError("person.deathdate", new LocalizedValidationError("validation.person.death_before_birth"));
				}
			}
		}
		
		if(errors.hasErrors()){
			throw errors;
		}
	}
	// ==========================================================
	// === Helper Methods
	// ==========================================================
	public List<String> searchCountries(String iValue)
	{
		/*
		// to test loading
		try
        {
	        Thread.sleep(7000);
        }
        catch(InterruptedException e)
        {
	        // TODO Auto-generated catch block
	        e.printStackTrace();
        }
        */
		
//		int lastStartsWith = 0;
		String value = iValue.trim().toLowerCase();
		
		if(value.length() == 0){
			return null;
		}
		
		List<String> matchingCountries = new ArrayList<String>();
		
		for(int i=0; i<allCountries.length; i++)
		{
			/*
			int idx = allCountriesLC[i].indexOf(val);
			if(idx == 0)
			{
				// --- starts with: in the first part of the list
				matchingCountries.add(lastStartsWith, allCountries[i]);
				lastStartsWith++;
			}
			else if(idx > 0)
			{
				// --- contains: in the second half
				matchingCountries.add(allCountries[i]);
			}
			*/
		    
			if(allCountries[i].regionMatches(true, 0, value, 0, value.length())){
				matchingCountries.add(allCountries[i]);
			}
		}
		return matchingCountries;
	}
	public boolean isGenderChosen()
	{
		return gender != 0;
	}
	public boolean isFemale()
	{
		return gender == 1;
	}
	// ==========================================================
	// === Bean Getters/Setters
	// ==========================================================
	public static int[] getAllGenders()
	{
		return ALL_GENDERS;
	}
	public static int[] getAllAchievements()
	{
		return ALL_ACHIEVEMENTS;
	}
	public int getGender()
	{
		return gender;
	}
	public void setGender(int gender)
	{
		this.gender = gender;
	}
	public String getFirstname()
	{
		return firstname;
	}
	public void setFirstname(String firstname)
	{
		this.firstname = firstname;
	}
	public String getLastname()
	{
		return lastname;
	}
	public void setLastname(String lastname)
	{
		this.lastname = lastname;
	}
	public String getMaidenname()
	{
		return maidenname;
	}
	public void setMaidenname(String maidenname)
	{
		this.maidenname = maidenname;
	}
	public Date getBirthdate()
	{
		return birthdate;
	}
	public void setBirthdate(Date birthdate)
	{
		this.birthdate = birthdate;
	}
	public Date getDeathdate()
	{
		return deathdate;
	}
	public void setDeathdate(Date deathdate)
	{
		this.deathdate = deathdate;
	}
	public int getBirthsize()
	{
		return birthsize;
	}
	public void setBirthsize(int birthsize)
	{
		this.birthsize = birthsize;
	}
	public int getBirthweight()
	{
		return birthweight;
	}
	public void setBirthweight(int birthweight)
	{
		this.birthweight = birthweight;
	}
	public String getBirthcountry()
	{
		return birthcountry;
	}
	public void setBirthcountry(String birthcountry)
	{
		this.birthcountry = birthcountry;
	}
	public String getBirthcity()
	{
		return birthcity;
	}
	public void setBirthcity(String birthcity)
	{
		this.birthcity = birthcity;
	}
	public String getDeathcity()
	{
		return deathcity;
	}
	public String getDeathcountry()
	{
		return deathcountry;
	}
	public void setDead(boolean dead)
	{
		this.dead = dead;
	}
	public boolean isDead()
	{
		return dead;
	}
	public void setDeathcity(String deathcity)
	{
		this.deathcity = deathcity;
	}
	public void setDeathcountry(String deathcountry)
	{
		this.deathcountry = deathcountry;
	}
	public int[] getAchievements()
	{
		return achievements;
	}
	public void setAchievements(int[] achievements)
	{
		if(achievements != null){
			this.achievements = achievements.clone();
		}
	}
	
	
}
