/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.samples.livetour.uibeans;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Locale;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.opensuit.core.session.OpenSuitSession;
import org.ow2.opensuit.xmlmap.utils.XML2HTML;
import org.xml.sax.SAXException;


public class UserConfig implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	public void logout(HttpServletRequest request)
	{
		request.getSession().invalidate();
	}
	public void viewXml2Html(HttpServletRequest request, HttpServletResponse response, String file) throws ParserConfigurationException, SAXException, IOException
	{
		InputStream input = request.getSession().getServletContext().getResourceAsStream(file);
		
		// --- render response
		response.setContentType("text/html");
		response.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		response.setHeader("Pragma", "no-cache"); //HTTP 1.0
		response.setDateHeader("Expires", 0); //prevents caching at the proxy server
		
		XML2HTML xml2html = new XML2HTML();
		xml2html.format(input, response.getOutputStream());
	}
	public void viewXml(HttpServletRequest request, HttpServletResponse response, String file) throws ParserConfigurationException, SAXException, IOException
	{
		InputStream input = request.getSession().getServletContext().getResourceAsStream(file);
		OutputStream output = response.getOutputStream();
		
		// --- render response
		response.setContentType("text/xml");
		response.setHeader("Cache-Control", "no-cache"); //HTTP 1.1
		response.setHeader("Pragma", "no-cache"); //HTTP 1.0
		response.setDateHeader("Expires", 0); //prevents caching at the proxy server
		
		byte[] buffer = new byte[512];
		int len = 0;
		while((len = input.read(buffer)) >= 0)
		{
			output.write(buffer, 0, len);
		}
		output.flush();
		output.close();
	}
	public Locale getLocale()
	{
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		Cookie[] cookies = request.getCookies();
		for(int i=0; i<cookies.length; i++)
		{
			if(cookies[i].getName().equals("lang"))
			{
//				System.out.println("cookie found: "+cookies[i].getValue());
				return new Locale(cookies[i].getValue());
			}
		}
		
		// --- otherwise return default locale
//		System.out.println("cookie not found - use default language: "+request.getRequestURL().toString());
		return OpenSuitSession.getCurrentApplication().getAllLanguages()[0].getLocale();
	}
	public void setLocale(Locale locale)
	{
		HttpServletRequest request = OpenSuitSession.getCurrentRequest();
		HttpServletResponse response = OpenSuitSession.getCurrentResponse();
		Cookie langCookie = new Cookie("lang", locale.getLanguage());
		langCookie.setPath(request.getContextPath());
		response.addCookie(langCookie);
	}
}
